@@ -1,7 +1,9 @@
-/* display.c, Ait, BSD 3-Clause, Kevin Bloom, 2023-2024,
+/* display.c, Ait, BSD 3-Clause, Kevin Bloom, 2023-2025,
    Derived from: Atto January 2017
    Derived from: Anthony's Editor January 93
 */
+
+#include <sys/types.h>
 #include "header.h"
 #include "util.h"
 #define TB_IMPL
@@ -108,7 +110,7 @@ point_t lncolumn(buffer_t *bp, point_t offset, int column)
 int syntax_color(buffer_t *bp)
 {
   int color = TB_DEFAULT;
-  int token_type = parse_text(bp, bp->b_epage);
+  int token_type = parse_text(bp, bp->b_epage, FALSE);
   switch(token_type) {
     case ID_SYMBOL: {
       color = TB_MAGENTA;
@@ -267,6 +269,7 @@ void display(window_t *wp, int flag)
       } else if(matching_bracket >= 0 && bp->b_epage == matching_bracket) {
         matched = 1;
       } else if(curwp == wp) {
+        char t = searchtext[0];
         if(found_point > 0 &&
            search_dir == 1 &&
            found_point - searchlen <= bp->b_epage &&
@@ -277,11 +280,12 @@ void display(window_t *wp, int flag)
                 found_point + searchlen >= bp->b_epage &&
                 bp->b_epage > found_point)
           region = 1;
-        else if(found_point > 0 && tolower(searchtext[0]) == tolower(*p) && b == 0) {
-          if(isupper(searchtext[0]))
+        else if(found_point > 0 && tolower(t) == tolower(*p) && b == 0) {
+          if(isupper(t))
             case_sense = TRUE;
           while(b < searchlen) {
-            if(isupper(searchtext[b])) {
+            t = searchtext[b];
+            if(isupper(t)) {
               case_sense = TRUE;
               if(searchtext[b] == *(p+b)) {
                 region = 1;
@@ -320,6 +324,8 @@ void display(window_t *wp, int flag)
       if(!isspace(*p) || *p == '\n' || *p > 32) {
         int color = syntax_color(bp);
         fg = region ? TB_DEFAULT : color;
+      } else {
+        dec_skip();
       }
       bg = region ? TB_WHITE : TB_DEFAULT;
       if(nch > 1) {
@@ -409,10 +415,10 @@ void display(window_t *wp, int flag)
                wp->w_update) {
               send_attr(fg, mc);
               send_char(j-1, i, uni);
-              cell_set(front_wide, &uni, 1, fg, mc);
-              cell_set(back_wide, &uni, 1, fg, mc);
+              tb_set_cell(j-1, i, uni, fg, mc);
+//               cell_set(front_wide, &uni, 1, fg, mc);
+//               cell_set(back_wide, &uni, 1, fg, mc);
             }
-//           tb_set_cell(j-1, i, uni, fg, mc);
           }
         }
         if(j > max_col) {
@@ -478,7 +484,7 @@ void display(window_t *wp, int flag)
   b2w(wp); /* save buffer stuff on window */
   modeline(wp);
   if (wp == curwp && flag) {
-    dispmsg();
+   dispmsg();
     tb_set_cursor(bp->b_col, bp->b_row); /* set cursor */
   }
   wp->w_update = FALSE;
@@ -581,19 +587,22 @@ void dispmsg()
     addstr(msgline);
     msgflag = FALSE;
   }
-  clrtoeol(msgline, MSGLINE);
+  if(msgline[0] != '\0')
+    clrtoeol((int)strlen(msgline), MSGLINE);
 }
 
 void display_prompt_and_response(char *prompt, char *response)
 {
   char *full;
   asprintf((char **)&full, "%s%s", prompt, response);
-  full[strlen(full)] = '\0';
+  int size = strlen(full);
   print_to_msgline(prompt);
   /* if we have a value print it and go to end of it */
   if (response[0] != '\0')
     addstr(response);
-  clrtoeol(full, MSGLINE);
+  clrtoeol(size, MSGLINE);
+  free(full);
+  full = NULL;
 }
 
 void update_display()
@@ -744,23 +753,22 @@ void addstrxy(char *c, int x, int y, int region)
   printf_tb(x, y, fg, bg, "%s", c);
 }
 
-void clrtoeol(const char *str, int line)
+void clrtoeol(int len, int line)
 {
-  int l = strlen(str);
-  int size = COLS-l;
+  int size = COLS-len;
   if(size < 0)
     return;
-  char blanks[size], entire[COLS];
-
+  char *blanks = NULL;
   if(size == 0) {
-    for(int i=0; i<size; i++) {
-      entire[i]=' ';
-    }
-    printf_tb(l, line, TB_DEFAULT, TB_DEFAULT, "%s", entire);
+    blanks = malloc((COLS+1)*sizeof(char));
+    memset(blanks, ' ', COLS);
+    blanks[COLS] = '\0';
   } else {
-    for(int i=0; i<size; i++) {
-      blanks[i]=' ';
-    }
-    printf_tb(l, line, TB_DEFAULT, TB_DEFAULT, "%s", blanks);
-  }
+    blanks = malloc((size+1)*sizeof(char));
+    memset(blanks, ' ', size);
+    blanks[size] = '\0';
+   }
+  printf_tb(len, line, TB_DEFAULT, TB_DEFAULT, "%s", blanks);
+  free(blanks);
+  blanks = NULL;
 }
