@@ -1,4 +1,4 @@
-/* gap.c, Ait, BSD 3-Clause, Kevin Bloom, 2023-2024,
+/* gap.c, Ait, BSD 3-Clause, Kevin Bloom, 2023-2025,
    Derived from: Atto January 2017
    Derived from: Anthony's Editor January 93
 */
@@ -281,50 +281,41 @@ void undoset(int type, int shouldconcat)
       }
       break;
     case DELETE:
+      if(curbp->b_buf == NULL)
+        return;
       npc = utf8_size(*ptr(curbp,curbp->b_point));
       if(curbp->b_undo != NULL && curbp->b_undo->u_type == DELETE && shouldconcat) {
-//         ulen = strlen((const char *)curbp->b_undo->u_data);
-        dtemp = (char_t *)malloc(npc);
-        (void) memcpy(dtemp, ptr(curbp, curbp->b_point), npc);
-//         dtemp[npc] = '\0';
-//         utemp = (char_t *) strndup((const char *)curbp->b_undo->u_data, ulen);
-//         curbp->b_undo->u_data = (char_t *)(realloc(curbp->b_undo->u_data, (size_t)(npc + ulen)));
+        dtemp = (char_t *) strndup((const char *)ptr(curbp, curbp->b_point), npc);
         asprintf((char **)&utemp, "%s%s", curbp->b_undo->u_data, dtemp);
-//         curbp->b_undo->u_data[npc+ulen] = '\0';
         free(curbp->b_undo->u_data);
         curbp->b_undo->u_data = utemp;
         free(dtemp);
-//         free(utemp);
         dtemp = NULL;
-//         utemp = NULL;
         return;
-      } else {
-        u->u_data = (char_t *)(malloc(npc));
+      } else if(npc > 0) {
+        u->u_data = (char_t *)(malloc(npc+1));
         memcpy(u->u_data, ptr(curbp, curbp->b_point), npc);
         u->u_data[npc] = '\0';
       }
       break;
     case BACKSP:
+      if(curbp->b_buf == NULL)
+        return;
       npc = prev_utf8_char_size();
       if(curbp->b_undo != NULL && curbp->b_undo->u_type == BACKSP && shouldconcat) {
          curbp->b_undo->u_point -= npc;
-         dtemp = (char_t *)malloc(npc);
+         dtemp = (char_t *)malloc(npc+1);
          (void) memcpy(dtemp, ptr(curbp, curbp->b_undo->u_point), npc);
          dtemp[npc] = '\0';
-//          ulen = strlen((const char *)curbp->b_undo->u_data);
-//          utemp = (char_t *) strndup((const char *)curbp->b_undo->u_data, ulen);
-//          curbp->b_undo->u_data = (char_t *)(realloc(curbp->b_undo->u_data, (size_t)(npc + ulen)));
          asprintf((char **)&utemp, "%s%s", dtemp, curbp->b_undo->u_data);
-//          curbp->b_undo->u_data[(npc + ulen)] = '\0';
          free(curbp->b_undo->u_data);
          curbp->b_undo->u_data = utemp;
          free(dtemp);
-//          utemp = NULL;
          dtemp = NULL;
          return;
       } else {
         u->u_point = curbp->b_point - npc;
-        u->u_data = (char_t *)(malloc(npc));
+        u->u_data = (char_t *)(malloc(npc+1));
         strncpy((char *)u->u_data, (const char *)ptr(curbp, u->u_point), npc);
         u->u_data[npc] = '\0';
         u->u_adjust = TRUE;
@@ -347,9 +338,7 @@ void undoset(int type, int shouldconcat)
       if(curbp->b_undo != NULL &&
          curbp->b_undo->u_type == CUT &&
          shouldconcat) {
-//         int npc = strlen((char *)curbp->b_undo->u_data);
-//         utemp = (char_t *) strndup((const char *)curbp->b_undo->u_data, npc);
-        dtemp = malloc(length);
+        dtemp = malloc(length+1);
         memcpy(dtemp, p, length);
         dtemp[length] = '\0';
         if(shouldconcat < 0) {
@@ -363,7 +352,6 @@ void undoset(int type, int shouldconcat)
             (char **)&utemp,
             "%s%s", curbp->b_undo->u_data, dtemp
           );
-//         curbp->b_undo->u_data[length+npc] = '\0';
         if(curbp->b_undo->u_data != NULL)
           free(curbp->b_undo->u_data);
         curbp->b_undo->u_data = utemp;
@@ -378,7 +366,6 @@ void undoset(int type, int shouldconcat)
               break;
             }
         }
-//         free(utemp);
         free(dtemp);
         return;
       } else {
@@ -407,21 +394,21 @@ void undoset(int type, int shouldconcat)
       (void) movegap(curbp, curbp->b_point);
       p = ptr(curbp, curbp->b_point);
       length = curbp->b_mark - curbp->b_point;
-      u->u_data = (char_t*) malloc(length);
+      u->u_data = (char_t*) malloc(length+1);
       (void) memcpy(u->u_data, p, length);
       u->u_data[length] = '\0';
       u->u_size = shouldconcat;
       break;
     case CLIPBOARD: {
       int ntemp = strlen(gtemp);
-      u->u_data = (char_t*) malloc(ntemp);
+      u->u_data = (char_t*) malloc(ntemp+1);
       (void) memcpy(u->u_data, gtemp, ntemp);
       u->u_data[ntemp] = '\0';
       break;
     }
     case LOAD: {
       (void) movegap(curbp, (point_t) 0);
-      u->u_data = (char_t*) malloc(curbp->b_size);
+      u->u_data = (char_t*) malloc(curbp->b_size+1);
       (void) memcpy(u->u_data, curbp->b_egap, curbp->b_size);
       u->u_data[curbp->b_size] = '\0';
       u->u_size = curbp->b_size;
@@ -470,7 +457,7 @@ void redo_or_undo_set(undo_t *up, int datalen, int isundo)
     rp->u_type = REPLACE;
     (void) movegap(curbp, up->u_point);
     p = ptr(curbp, up->u_point);
-    rp->u_data = (char_t*) malloc(up->u_size);
+    rp->u_data = (char_t*) malloc(up->u_size+1);
     (void) memcpy(rp->u_data, p, up->u_size);
     rp->u_data[up->u_size] = '\0';
     rp->u_size = datalen;
@@ -478,7 +465,7 @@ void redo_or_undo_set(undo_t *up, int datalen, int isundo)
   case LOAD:
     rp->u_type = LOAD;
     (void) movegap(curbp, (point_t) 0);
-    rp->u_data = (char_t*) malloc(curbp->b_size);
+    rp->u_data = (char_t*) malloc(curbp->b_size+1);
     (void) memcpy(rp->u_data, curbp->b_egap, curbp->b_size);
     rp->u_data[curbp->b_size] = '\0';
     rp->u_size = curbp->b_size;
@@ -495,7 +482,7 @@ void redo_or_undo_set(undo_t *up, int datalen, int isundo)
   }
   /* if an undo was done, save to redo */
   if(isundo) {
-     rp->u_next = curbp->b_redo;
+    rp->u_next = curbp->b_redo;
     curbp->b_redo = rp;
   } else {
     rp->u_next = curbp->b_undo;
@@ -539,7 +526,6 @@ void undo_or_redo(buffer_t *bp, undo_t *up, int isundo)
     (void) movegap(bp, bp->b_point);
     bp->b_egap += up->u_size;
     bp->b_point = pos(bp, bp->b_egap);
-
     bp->b_point = movegap(bp, bp->b_point);
     memcpy(bp->b_gap, up->u_data, n);
     bp->b_gap += n;
@@ -619,6 +605,7 @@ void get_line_stats(int *curline, int *lastline, buffer_t *bp)
   *curline = -1;
 
   for (p=0, line=0; p < end_p; p++) {
+
     line += (*(ptr(bp,p)) == '\n') ? 1 : 0;
     *lastline = line;
 
@@ -651,27 +638,28 @@ int file_was_modified_prompt()
 {
   int current = 0, lastln = 0;
   const char *prompt = "This file has changed on disk; really edit (y/N/r) ?";
+  int size = strlen(prompt);
   char c = '\0';
   point_t p = curbp->b_point;
   struct stat sb;
   print_to_msgline(prompt);
-  clrtoeol(prompt, MSGLINE);
+  clrtoeol(size, MSGLINE);
   c = yesnomaybeso('n');
   if (c == 'n') {
-    clrtoeol("", MSGLINE);
+    clrtoeol(0, MSGLINE);
     return FALSE;
   } else if(c == 'r') {
     curbp->b_point = 0;
     undoset(LOAD, FALSE);
     load_file(curbp->b_fname);
-    clrtoeol("", MSGLINE);
+    clrtoeol(0, MSGLINE);
     curbp->b_point = p;
     get_line_stats(&current, &lastln, curbp);
     curbp->b_line = current;
     msg("Buffer reverted.");
     return FALSE;
   } else if(c == 'y') {
-    clrtoeol("", MSGLINE);
+    clrtoeol(0, MSGLINE);
     if (stat(curbp->b_fname, &sb) < 0) {
       msg("Failed to find file \"%s\".", curbp->b_fname);
       return (FALSE);
@@ -683,6 +671,6 @@ int file_was_modified_prompt()
     curbp->b_fmtime = sb.st_mtime;
     return TRUE;
   }
-  clrtoeol("", MSGLINE);
+  clrtoeol(0, MSGLINE);
   return (FALSE);
 }
