@@ -1,8 +1,9 @@
-/* key.c, Ait, BSD 3-Clause, Kevin Bloom, 2023-2024,
+/* key.c, Ait, BSD 3-Clause, Kevin Bloom, 2023-2025,
    Derived from: Atto January 2017
    Derived from: Anthony's Editor January 93
 */
 
+#include <sys/types.h>
 #include "header.h"
 #include "termbox.h"
 #include "util.h"
@@ -181,10 +182,12 @@ char_t *get_key(keymap_t *keys, keymap_t **key_return)
     if(!execute_kbd_macro) {
       int ci = 0, size = 1;
       if(gtemp == NULL) {
-        gtemp = malloc(sizeof(char) * TEMPBUF);
+        gtemp = calloc(TEMPBUF+1, sizeof(char));
+        ngtemp = TEMPBUF+1;
       }
       unicode_buf[tb_utf8_unicode_to_char(unicode_buf, ev.ch)] = '\0';
-      gtemp[ci] = ev.key == TB_KEY_CTRL_M ? '\n' : unicode_buf[0];
+      gtemp[ci] = ev.key == TB_KEY_CTRL_M || ev.key == TB_KEY_CTRL_J ?
+         '\n' : unicode_buf[0];
       ret = tb_peek_event(&ev1, 0);
       if(record_input) {
         record_buffer[record_buffer_index] = ev1;
@@ -192,13 +195,27 @@ char_t *get_key(keymap_t *keys, keymap_t **key_return)
       }
       ci++;
       while(ret == TB_OK) {
-        if(ci > TEMPBUF*size) {
-          size++;
-          gtemp = realloc(gtemp, TEMPBUF*size);
+        int maxsize = TEMPBUF*size;
+        if(ci >= maxsize) {
+          char *p2;
+          int newsize = TEMPBUF*(++size)+1;
+          if ((p2 = calloc(newsize, sizeof(char))) == NULL) {
+            if (gtemp != NULL)
+              free(gtemp);
+            gtemp = NULL;
+            return NULL;
+          }
+          memcpy(p2, gtemp, ngtemp);
+          free(gtemp);
+          gtemp = NULL;
+          gtemp = p2;
+          ngtemp = newsize;
         }
         int uni_size = tb_utf8_unicode_to_char(unicode_buf, ev1.ch);
         unicode_buf[uni_size] = '\0';
-        unicode_buf[0] = ev1.key == TB_KEY_CTRL_M ? '\n' : unicode_buf[0];
+        unicode_buf[0] =
+          ev1.key == TB_KEY_CTRL_M || ev1.key == TB_KEY_CTRL_J ?
+            '\n' : unicode_buf[0];
         strncat(gtemp, unicode_buf, uni_size);
         ci += uni_size;
         ret = tb_peek_event(&ev1, 0);
@@ -208,7 +225,7 @@ char_t *get_key(keymap_t *keys, keymap_t **key_return)
         }
       }
       gtemp[ci] = '\0';
-      if(strlen(gtemp) > 2) {
+      if(ci > 2) {
         k = extraneous_keymap;
         *key_return = k;
         record = buffer;
@@ -265,7 +282,7 @@ int getinput(char *prompt, char *buf, int nbuf, int flag, int allowblank)
   struct tb_event ev;
 
   print_to_msgline(prompt);
-  clrtoeol(prompt, MSGLINE);
+  clrtoeol(start_col, MSGLINE);
 
   if (flag == F_CLEAR) buf[0] = '\0';
 
@@ -315,6 +332,7 @@ int getinput(char *prompt, char *buf, int nbuf, int flag, int allowblank)
         return (cpos > 0 ? TRUE : allowblank);
       }
       case TB_KEY_CTRL_G: /* ctrl-g */
+        clrtoeol(0, MSGLINE);
         return FALSE;
 
       case TB_KEY_BACKSPACE2: /* del, erase */
@@ -390,9 +408,9 @@ int msgline_editor(
         strcat(full, buf);
         tb_set_cursor(start_col, MSGLINE);
         addstr(buf);
-        clrtoeol(full, MSGLINE);
+        clrtoeol(start_col + *cpos, MSGLINE);
         tb_set_cursor(start_col + *cpos, MSGLINE);
-        full[strlen(prompt)] = '\0';
+        full[start_col + *cpos] = '\0';
         return 1;
       } else if(k == 'y') {
         int ondef = FALSE, pastdef = FALSE;
@@ -409,14 +427,24 @@ int msgline_editor(
             ii--;
           }
         }
-        buf[*cpos-3] = '\0';
-        *cpos -= 3;
+        /* Since the default value can contain anything, we have to
+           determine we're at the end of the default value we go all
+           the way to the null terminator. Once there, we know that
+           there is always a two char postfix of "):" To not copy
+           this into the msgline buffer, we delete those by setting
+           them to null terminator.
+
+           We have to set i to 3 here because of cpos being 1 ahead
+           of the length everything.
+        */
+        for(int i = 3; i > 0; i--)
+          buf[--*cpos] = '\0';
         strcat(full, buf);
         tb_set_cursor(start_col, MSGLINE);
         addstr(buf);
-        clrtoeol(full, MSGLINE);
+        clrtoeol(start_col + *cpos, MSGLINE);
         tb_set_cursor(start_col + *cpos, MSGLINE);
-        full[strlen(prompt)] = '\0';
+        full[start_col + *cpos] = '\0';
         return 1;
       } else if(ev.key == TB_KEY_BACKSPACE || ev.key == TB_KEY_BACKSPACE2) {
         if ((!isspace(buf[*cpos-1]) || !is_symbol(buf[*cpos-1])) && *cpos != 0) {
@@ -440,9 +468,9 @@ int msgline_editor(
         strcat(full, buf);
         tb_set_cursor(start_col, MSGLINE);
         addstr(buf);
-        clrtoeol(full, MSGLINE);
+        clrtoeol(start_col + *cpos, MSGLINE);
         tb_set_cursor(start_col + *cpos, MSGLINE);
-        full[strlen(prompt)] = '\0';
+        full[start_col + *cpos] = '\0';
         return 1;
       }
     } else
@@ -476,7 +504,7 @@ int msgline_editor(
       for(i = *cpos; buf[i] != '\0'; i++)
         buf[i] = '\0';
       strcat(full, buf);
-      clrtoeol(full, MSGLINE);
+      clrtoeol(start_col + *cpos, MSGLINE);
       return 1;
 
     case TB_KEY_DELETE:
@@ -489,9 +517,9 @@ int msgline_editor(
       strcat(full, buf);
       tb_set_cursor(start_col, MSGLINE);
       addstr(buf);
-      clrtoeol(full, MSGLINE);
+      clrtoeol(start_col + strlen(buf), MSGLINE);
       tb_set_cursor(start_col + *cpos, MSGLINE);
-      full[strlen(prompt)] = '\0';
+      full[start_col + *cpos] = '\0';
       return 1;
     case TB_KEY_BACKSPACE2: /* del, erase */
     case TB_KEY_BACKSPACE: /* backspace */
@@ -504,15 +532,36 @@ int msgline_editor(
       strcat(full, buf);
       tb_set_cursor(start_col, MSGLINE);
       addstr(buf);
-      clrtoeol(full, MSGLINE);
+      clrtoeol(start_col + *cpos, MSGLINE);
       tb_set_cursor(start_col + *cpos, MSGLINE);
-      full[strlen(prompt)] = '\0';
+      full[start_col + *cpos] = '\0';
       return 1;
 
+    case TB_KEY_CTRL_U: {
+      universal_argument++;
+      return 1;
+    }
+
     case TB_KEY_CTRL_Y: {
+      char_t *oscrap, *new_scrap = NULL;
+      int onscrap = 0;
+
       if(scrap.data == NULL)
         return 0;
 
+      if(universal_argument > 0 && universal_argument-1 < KILLRING_SIZE) {
+        oscrap = (char_t *)strndup((char *)scrap.data, scrap.len);
+        onscrap = scrap.len;
+        free(scrap.data);
+        scrap.len = kill_ring[universal_argument-1].len;
+        new_scrap = (char_t *) strndup(
+          (const char *)kill_ring[universal_argument-1].data,
+          scrap.len
+        );
+        scrap.data = new_scrap;
+        universal_argument = 0;
+      }
+
       int remaining = nbuf - strlen(buf) - 1;
       int size = scrap.len > remaining ? remaining : scrap.len;
       *cpos += size;
@@ -522,9 +571,14 @@ int msgline_editor(
       strncat(full, buf, size);
       tb_set_cursor(start_col, MSGLINE);
       addstr(buf);
-      clrtoeol(full, MSGLINE);
+      clrtoeol(start_col + *cpos, MSGLINE);
       tb_set_cursor(start_col + *cpos, MSGLINE);
-      full[strlen(prompt)] = '\0';
+      full[start_col + *cpos] = '\0';
+      if(onscrap > 0) {
+        free(scrap.data);
+        scrap.len = onscrap;
+        scrap.data = oscrap;
+      }
       return 1;
     }
     case TB_KEY_CTRL_Q: {
